//
// <PW-Id>
//

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// Copyright 2018-2020 by Pierre Wolfers Meylan France                       //
//                                                                           //
// This library is free software. Distribution and use rights are outlined   //
// in the file "COPYING" which should have been included with this file.     //
// If this file is missing or damaged, see the license at:                   //
//                                                                           //
//    <To be define when ready>                                              //
//                                                                           //
//   This license described in this file overrides all other licenses that   //
//   might be specified in other files for this library.                     //
//                                                                           //
//   This library is free software; you can redistribute it  and/or modify   //
//   it under the terms of the GNU Lesser General Public License as publi-   //
//   shed by  the  Free Software  Foundation;  either  version 2.1  of the   //
//   License, or (at your option) any later version.                         //
//                                                                           //
//   This library  is  distributed in  the  hope that  it will  be useful,   //
//   but   WITHOUT  ANY  WARRANTY;  without even  the  implied warranty of   //
//   MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the GNU   //
//   Library General Public License for more details.                        //
//                                                                           //
//   You should have  received  a copy of  the  GNU Lesser General  Public   //
//   License  along with this library  (see COPYING.LIB); if not, write to   //
//   the Free Software Foundation :                                          //
//                        Inc., 675 Mass Ave, Cambridge, MA 02139, USA.      //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



//
// P.Wolfers Software
//

//
// Environnement fto generatea specific FLTK Spinner Widget.
// P.Wolfers
//



#include <stdio.h>

#include "FlW_Spinner.h"


void FlW_Spinner::sb_cb(  Fl_Widget * w, FlW_Spinner * sb )
{
    double v;

    if (w == &(sb->input_)) {
        v = atof( sb->input_.value());
        if (v < sb->min_) {
            sb->val_ = sb->min_; sb->update();
        } else if (v > sb->max_) {
            sb->val_ = sb->max_; sb->update();
        } else sb->val_ = v;
    } else if (w == &(sb->upcmd_)) {
        v = sb->val_ + sb->stp_;
        if (v > sb->max_) sb->val_ = sb->max_;
                     else sb->val_ = v;
        sb->update();
    } else if (w == &(sb->dncmd_)) {
        v = sb->val_ - sb->stp_;
        if (v > sb->min_) sb->val_ = sb->min_;
                     else sb->val_ = v;
        sb->update();
    }
    sb->set_changed();
    sb->do_callback();
} // void FlW_Spinner::sb_cb(  Fl_Widget *w, FlW_Spinner *sb ).



FlW_Spinner::FlW_Spinner( int X, int Y, int W, int H, const char * L ) :
                        Fl_Group( X, Y, W, H, L ),
                        input_( X,                     Y, W - H/2 - 2,   H ),
                        upcmd_( X + W - H/2 - 2,       Y,     H/2 + 2, H/2, "@-42<" ),
                        dncmd_( X + W - H/2 - 2, Y + H/2,     H/2 + 2, H/2, "@-42>" )
{
    val_ = 0.0; stp_ =   1.0;
    min_ = 0.0; max_ = 100.0;
    frm_ = "%g";

    outcolor_ = FL_RED;
    actcolor_ = FL_BACKGROUND_COLOR;

    align(FL_ALIGN_LEFT);
  
    input_.value("0");
    input_.type(FL_INT_INPUT);
    input_.when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);

    input_.callback((Fl_Callback *)sb_cb, this);
    upcmd_.callback((Fl_Callback *)sb_cb, this);
    dncmd_.callback((Fl_Callback *)sb_cb, this);

} // FlW_Spinner::FlW_Spinner( int X, int Y, int W, int H, const char * L ).



void FlW_Spinner::update()
{
    char s[255];

    if ((frm_[0] == '%')&&(frm_[1] == '.') && (frm_[2] == '*')) {
        // Precision argument.
        // This code block is a simplified version of ...
        // ... Fl_Valuator::format() and works well (but looks ugly).
        int c = 0;
        char temp[64], * sp = temp;

        sprintf( temp, "%.12f", stp_ );
        while (*sp) sp++; sp--;
        while ((sp > temp) && (*sp == '0')) sp--;
        while ((sp > temp) && (*sp >= '0') && (*sp <= '9')) { sp--; c++; }
        sprintf( s, frm_, c, val_ );
    } else sprintf( s, frm_, val_ );
    input_.value( s );
} // void FlW_Spinner::update().



int  FlW_Spinner::handle( int event )
{
    switch (event) {
        case FL_KEYDOWN :
        case FL_SHORTCUT :
            if (Fl::event_key() == FL_Up) {
                upcmd_.do_callback();
                return 1;
            } else if (Fl::event_key() == FL_Down) {
                dncmd_.do_callback();
                return 1;
            } else return 0;

        case FL_FOCUS :
            if (input_.take_focus()) return 1;
                                else return 0;
    }
    return Fl_Group::handle(event);
} //int  FlW_Spinner::handle( int event ).



void FlW_Spinner::resize( int X, int Y, int W, int H )
{
    Fl_Group::resize( X, Y, W, H );
    input_.resize(               X,       Y, W - H/2 - 2,   H );
    upcmd_.resize( X + W - H/2 - 2,       Y,     H/2 + 2, H/2 );
    upcmd_.resize( X + W - H/2 - 2, Y + H/2,     H/2 + 2, H/2 );
} // void FlW_Spinner::resize( int X, int Y, int W, int H ).



void FlW_Spinner::step( double s )
{
    stp_ = s;
    if (stp_ != (int)stp_) input_.type(FL_FLOAT_INPUT);
                      else input_.type(FL_INT_INPUT);
    update();
} // void FlW_Spinner::step( double f ).



void FlW_Spinner::set_output()
{
    FlW_Spinner::upcmd_.deactivate();
    FlW_Spinner::dncmd_.deactivate();
    FlW_Spinner::input_.set_output();
    FlW_Spinner::input_.color( outcolor_ );
} // void FlW_Spinner::set_output().



void FlW_Spinner::clear_output()
{
    FlW_Spinner::upcmd_.activate();
    FlW_Spinner::dncmd_.activate();
    FlW_Spinner::input_.clear_output();
    FlW_Spinner::input_.color( actcolor_ );    
} // void FlW_Spinner::clear_output().



//
// end of <PW-Id>.
//
