//
// <PW-Id>
//

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// Copyright 2018-2020 by Pierre Wolfers Meylan France                       //
//                                                                           //
// This library is free software. Distribution and use rights are outlined   //
// in the file "COPYING" which should have been included with this file.     //
// If this file is missing or damaged, see the license at:                   //
//                                                                           //
//    <To be define when ready>                                              //
//                                                                           //
//   This license described in this file overrides all other licenses that   //
//   might be specified in other files for this library.                     //
//                                                                           //
//   This library is free software; you can redistribute it  and/or modify   //
//   it under the terms of the GNU Lesser General Public License as publi-   //
//   shed by  the  Free Software  Foundation;  either  version 2.1  of the   //
//   License, or (at your option) any later version.                         //
//                                                                           //
//   This library  is  distributed in  the  hope that  it will  be useful,   //
//   but   WITHOUT  ANY  WARRANTY;  without even  the  implied warranty of   //
//   MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the GNU   //
//   Library General Public License for more details.                        //
//                                                                           //
//   You should have  received  a copy of  the  GNU Lesser General  Public   //
//   License  along with this library  (see COPYING.LIB); if not, write to   //
//   the Free Software Foundation :                                          //
//                        Inc., 675 Mass Ave, Cambridge, MA 02139, USA.      //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


//#define TD_DEBUG


//
// P.Wolfers Software
//


//
// Text Display module.
//


#include <stdio.h>
#include <stdarg.h>
#include <errno.h>
#include <string.h>

#include <FL/Fl.H>
#include <FL/fl_ask.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Native_File_Chooser.H>
#include <FL/Fl_Printer.H>


#include "MyText_Display.h"

#include "Text_DisplayUI.h"



MyText_Display * MyText_Display::Init( int w, int h, const char * name )
{
    MyText_Display * myt;

    Text_DisplayUI   * tui = new Text_DisplayUI( w, h, name );
    myt = tui->w_txtdspl;
    myt->tui_ = tui;
    return myt;
} // MyText_Display * MyText_Display::Init( int w, int h, const char * name ).



void MyText_Display::resize( int x, int y, int w, int h )
{
    Fl_Text_Display::resize( x, y, w, h );
    sc_ = sc_ = scrollbar_width();
    ww_ = w - sc_; hh_ = h - sc_;
} // MyText_Display::resize( int x, int y, int w, int h ).



int MyText_Display::handle( int event )
{
    static int mouse_st = 0;
    int      x1, y1, ie = 0;

    x1 = Fl::event_x();
    y1 = Fl::event_y();
    if ((x1 < ww_)&&(y1 < hh_)) {
        switch (event ) {
            case FL_PUSH:
                cx_ = x1; cy_ = y1;
                bt_ = Fl::event_button();
                mouse_st = 1;
                return 1;

            case FL_RELEASE:
                if (1||mouse_st) {
                    mouse_st = 0;
                    cx_ = x1;
                    cy_ = y1;
                    printf( " cx = %d, cy = %d\n", cx_, cy_ );
                    ipos_ = xy_to_position( cx_, cy_, CHARACTER_POS );
                    xy_to_rowcol( cx_, cy_, &currow_, &curcol_, CHARACTER_POS );
                    if (cbl_) cbl_( bt_, currow_, curcol_ );
                         else TextPosition();
                    return 1;
                }
            default: mouse_st = 0; return 1;
        }
    } else return Fl_Text_Display::handle( event );
} // MyText_Display::myhandle( int event );



MyText_Display::MyText_Display( int x, int y, int w, int h, const char * l ) : Fl_Text_Display( x, y, w, h, l )
{
    cbl_ = NULL;
    cx_ = cy_ = ipos_ = -1;
    currow_ = curcol_ = -1;
    buf_ = new Fl_Text_Buffer();
    buffer( buf_ );
    textfont( FL_COURIER );
    textsize( 12 );
} // MyText_Display::MyText_Display( int x, int y, int w, int h, const char * l ) : Fl_Text_Display( x, y, w, h, l ).



int MyText_Display::TextPosition()
{
    printf( " pos = %7d, Row = %4d, Col = %4d\n", ipos_, currow_, curcol_ );
    return ipos_;

} //int MyText_Display::TextPosition( int row&, int col& ).



void MyText_Display::SetFont( int fnt, int size )
{
    textfont( fnt ); textsize( size );
} // void MyText_Display::SetFont( int fnt, int size ).




void MyText_Display::TxtSave()
{
    char *  Text,
         * fpath;

    fpath = fl_file_chooser( "Save File as", "*.txt", "DiaViewer_Rank_Check.txt", 0 );

    if (fpath&&fpath[0]) {
        switch (buf_->savefile( fpath )) {
            case 1: // Open Error.
                fl_alert( "Error : Cannot create the file %s\n\t error message %s\n", fpath, strerror( errno ) );
                break;
            case 2: // Write Error.
                fl_alert( "Error :Write error to file %s\n\t error message %s\n", fpath, strerror( errno ) );
                break;
            default: ;
        }
    }
} // MyText_Display::TxtSave().



int  MyText_Display::Append( const char * fmt, ... )
{
    va_list ap;
    char buffer[1024];

    va_start( ap, fmt );
    ::vsnprintf( buffer, 1024, fmt, ap );
    va_end( ap );
    buf_->append( buffer );
    return buf_->length();
} // int  MyText_Display::Append( const char * fmt, ... ).






#ifdef WITH_DEBUG



void MouseCallBack( int bt, int row, int col )
{
    printf( " Clic with Button %d, at (raw,col) [%d,%d]\n", bt, row, col );
} // void MouseCallBack( int bt, int row, int col ).



int main( int argc, char ** argv )
{
    FILE      * fp;
    char line[256];
    char *     str;

    MyText_Display * DspTxt;

    if ((argc<=1)&&!argv[1]) {
        fl_alert( " No file was specified => stop" );
        return 1;
    }


    DspTxt = MyText_Display::Init( 800, 500, argv[1] );

    DspTxt->SetCallBack( (CallBack_t)MouseCallBack );

    if (fp = fl_fopen( argv[1], "r" )) {
        while (str = fgets( line, 255, fp )) {
            printf( " line = %s", line );
            // if (DspTxt->txt_buffer->text()) DspTxt->txt_buffer->text( str );
//          DspTxt->Append( "%s", str );
            DspTxt->append( str );
        }
        fclose( fp );

        Fl::run();

        printf( " Normal End.\n" );
    } else  {
        fl_alert( "The File %s cannot be opened", argv[1] );
        return 1;
    }
    return 0;
} // int main( int argc, char ** argv ).

#endif




//
// end of <PW-Id>.
//
