//
// <PW-Id>
//

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// Copyright 2018-2020 by Pierre Wolfers Meylan France                       //
//                                                                           //
// This library is free software. Distribution and use rights are outlined   //
// in the file "COPYING" which should have been included with this file.     //
// If this file is missing or damaged, see the license at:                   //
//                                                                           //
//    <To be define when ready>                                              //
//                                                                           //
//   This license described in this file overrides all other licenses that   //
//   might be specified in other files for this library.                     //
//                                                                           //
//   This library is free software; you can redistribute it  and/or modify   //
//   it under the terms of the GNU Lesser General Public License as publi-   //
//   shed by  the  Free Software  Foundation;  either  version 2.1  of the   //
//   License, or (at your option) any later version.                         //
//                                                                           //
//   This library  is  distributed in  the  hope that  it will  be useful,   //
//   but   WITHOUT  ANY  WARRANTY;  without even  the  implied warranty of   //
//   MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the GNU   //
//   Library General Public License for more details.                        //
//                                                                           //
//   You should have  received  a copy of  the  GNU Lesser General  Public   //
//   License  along with this library  (see COPYING.LIB); if not, write to   //
//   the Free Software Foundation :                                          //
//                        Inc., 675 Mass Ave, Cambridge, MA 02139, USA.      //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



//
// P.Wolfers Software
//

//
// DiaViewer GL picture Dialog functions.
//
//


#include <FL/Fl.H>
#include <FL/Fl_Hold_Browser.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Text_Display.H>
#include <FL/Fl_Native_File_Chooser.H>

#include <stdio.h>
#include <string.h>

#include "DiaViewer_SRV.h"
#include "DiaViewer_DCM.h"
#include "DiaViewer_GL.h"




const char * DLG_DirChooser( const char * msg, const char * fnm )
{
    static    char   path[512];
    char          *      npath;   
    int                     ie;

    if (NativeOFil) {   // Native directory chooser.
        Fl_Native_File_Chooser fnfc;
        fnfc.title( msg );
        fnfc.directory( fnm );
        fnfc.type( Fl_Native_File_Chooser::BROWSE_DIRECTORY );
        switch (ie = fnfc.show() ) {
            case -1: 
                fl_alert( "Error: %s", fnfc.errmsg() );
            case  1:
                return NULL;
            default:
                strncpy( path, fnfc.filename(), 511 );
        }
    } else {            // FLTK Directory file chooser.
        npath = fl_dir_chooser( msg, fnm, 0 /*Absolu path*/ );
        if (npath&&npath[0]) {
            strncpy( path, npath, 511 );
            return path;
        } else return NULL;
    }
    return path;
} // char * DLG_DirChooser( const char * msg, const char * fnm ).



/*
char * DLG_FileChoice( const char * msg, const char * pat, const char *fnm )
{
    // SetUp Parameter : NativeOFil = 1/0 for use Native_File_Chooser/FLTK_File_Chooser.
    if (NativeOFil) {
        Fl_Native_File_Chooser fnfc;
        fnfc.title( msg );
        fnfc.type(Fl_Native_File_Chooser::BROWSE_FILE);
        fnfc.filter( pat );
        fnfc.directory( fnm );           // default directory to use
        // Show native chooser
        switch ( fnfc.show() ) {
            case -1: // Error
                fl_message( "ERROR: %s\n", fnfc.errmsg());
                Return NULL;
            case  1: // Cancel
                return NULL;
            default: /File selected.
                return fnfc.filename());
        }
        
    } else {
        // We use the FLTK File Chooser.
        return fl_file_chooser ( msg, pat, fnm, 0  );
    }
} //  DLG_FileChoice( const char * msg, const char * pat, const char *fnm ).
*/

#define MAX_LINE 256


static int     brw_idx;
static void  * brw_dat;
static int     ret_sta;

static void brw_cb( Fl_Widget* o, void *dat )
// The Call back function called when the user clic ...
// on a brower line.
//
{
    brw_idx = ((Fl_Hold_Browser*)o)->value();
    brw_dat = ((Fl_Hold_Browser*)o)->data( brw_idx );
    ret_sta =   1;
}


int SlideVolSelect( const char * title, VolEntry * &vol )
{
    VolEntry        ** VolList;
    VolEntry         *  Volume;
    const char  * path, * vnam;
    int              len, nvol;

    char        line[MAX_LINE];
    static const
      char dnam[] = "<noname>";

    nvol = OpenVolTable.TabUse();       // Get number of opened volume.
    if (nvol <= 0) return -1;           // No opened volume : error!
    VolList = (VolEntry**) OpenVolTable.GetObjs();      // Get the list of opened volumes.

    ret_sta =    0;                     //
    brw_idx =    0;                     // Assume OK.
    brw_dat = NULL;

    Fl_Double_Window win( 500, 200, title );
    Fl_Hold_Browser *brw = new Fl_Hold_Browser( 5, 5, 490, 190, NULL );
    brw->callback( brw_cb );
    for(int i = 0; i < nvol; i++) {
        Volume = VolList[i];
        path = Volume->VRoot()->FName();
        vnam = Volume->VName();
        if (!(vnam&&vnam[0])) vnam = dnam;
        len  = strlen( path );
        len += strlen( vnam ) + 12;
        if (len>=MAX_LINE) {
            line[MAX_LINE-1] =   0;
            line[MAX_LINE-2] =   '.'; line[MAX_LINE-3] =   '.';
            line[MAX_LINE-4] =   '.'; line[MAX_LINE-5] =   ' ';
        }
        snprintf( line, MAX_LINE-1, " %3d / %s : %s", i+1, vnam, path );
        brw->add( line, (void*)Volume );
    }

    win.resizable( brw );
//  brw->make_visible();
//  brw->position( 0 );
    win.end();
    win.show();
    while(!ret_sta) Fl::wait();

//  printf( " -> Select %d\n", brw_idx );

    vol = (VolEntry*)brw_dat;
    
//  if (vol) printf( " Vol path = %s\n", vol->VRoot()->FName() );
    return brw_idx;
}


//
// end of <PW-Id>.
//

