//
// <PW-Id>
//

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// Copyright 2018-2020 by Pierre Wolfers Meylan France                       //
//                                                                           //
// This library is free software. Distribution and use rights are outlined   //
// in the file "COPYING" which should have been included with this file.     //
// If this file is missing or damaged, see the license at:                   //
//                                                                           //
//    <To be define when ready>                                              //
//                                                                           //
//   This license described in this file overrides all other licenses that   //
//   might be specified in other files for this library.                     //
//                                                                           //
//   This library is free software; you can redistribute it  and/or modify   //
//   it under the terms of the GNU Lesser General Public License as publi-   //
//   shed by  the  Free Software  Foundation;  either  version 2.1  of the   //
//   License, or (at your option) any later version.                         //
//                                                                           //
//   This library  is  distributed in  the  hope that  it will  be useful,   //
//   but   WITHOUT  ANY  WARRANTY;  without even  the  implied warranty of   //
//   MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the GNU   //
//   Library General Public License for more details.                        //
//                                                                           //
//   You should have  received  a copy of  the  GNU Lesser General  Public   //
//   License  along with this library  (see COPYING.LIB); if not, write to   //
//   the Free Software Foundation :                                          //
//                        Inc., 675 Mass Ave, Cambridge, MA 02139, USA.      //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



//
// P.Wolfers Software
//

//
// DiaViewer GL picture display manager.
//
//

#include "DiaViewer_GL.h"
#include "DiaViewer_SRV.h"
#include "DiaViewer_UI.h"
#include "DiaViewer_DIA.h"
#include "DiaViewer_DCM.h"
#include "DiaViewer_SUP.h"
#include "ImageMan_REF.h"

#include "DiaViewer_RkgUI.h"


#define GL_WNMAX     2

// #define GL_DEBUG




DiaViewer_GL  * ViewToPrint =     NULL; // Define the current Slide to print pointer.

char *          OpenVolPath =     NULL; // Default Open Volume Path.



// Define the windows pointers table (4 = Maximum value of GL_WNMAX windows).
static DiaViewer_UI *Viewer_UI[4] = { NULL, NULL, NULL, NULL };


int     screen_w,       screen_h,       // Identify the screen size in pixel ...
        screen_x,       screen_y;       // ... and position (not used but referenced).

// int  w_width =  860, w_high  =  715, // Size of window #0.
//      ViewerReady     =            0; // OpenGL window Init Flag.

static int ViewerNumber =            0; // Viewer_UI table Index of the new creating DiaViewer_UI/GL window.

static int                      svargc; // >To keep the task arguments number ...
static char                   **svargv; // ... and the task arguments list.

int     PrtFromGL       =            1; // Flag to select the print of Screen GL image.
                                        // It was a temporary default.

const char def_dtx[] = "No Folder Name",// Default output when no name/text are set ...
           def_vtx[] = "No Volume Name";// ... for Directories/Folders or Volumes.




void DiaWiewer_GL_Init( int argc, char **argv, int nw )
// Routine (function) to initialize the DiaViewer_GL (FLTK) windowing system. 
{
    static int init    =     1;
    DiaViewer_UI       *   win;
    char               * wname;

    // Allocate enough memory for window name change. 
    wname = new char[64];
    sprintf( wname, "DiaViewer Slide Window # %1d", nw+1 );

    // Get the size of the computer screen on the first call.
    if (init) {
        svargc = argc, svargv = argv;
        Fl::screen_xywh( screen_x, screen_y, screen_w, screen_h, screen_nb );
        init = 0;
    }

    // Create the First user interface.
    
    ViewerNumber = nw;

    win = new DiaViewer_UI( 0, 0, 860, 715, wname );

    win->show( argc, argv );   // Display the first window.

    Viewer_UI[nw] = win;
} // void DiaWiewer_GL_Init( int argc, char **argv, int nw ).



//
// Class DiaViewer_GL methodes.
//

DiaViewer_GL::DiaViewer_GL( int x, int y, int w, int h, const char *title )
            : Fl_Gl_Window( x, y, w, h, title )
// This creator is called by the DiaViewer_UI creator(when create the GL window.
{
    int ie;

    Win_Init            =            1; // To force GL context init on the first call of draw() method.
    Win_UInb            = ViewerNumber; // Get the DiaViewer_Ui index in DiaViewer_GL class.
    

    w_widt      =    w; w_high  =    h; // Default GL windows as 24*36 ??? Landscape Diapositive, ...
                                        // ... with a scale of ww/24 pixels/mm.
    // The initiale Diapositive is an empty diapositive (black).
    r_scale             =          1.0; // Just to initialize.
    ViewReady           =            0; // To force the GL and Viewer Initialisation.
    Curr_Map            =         NULL; // Initialize the current Pixel Map pointer.
    
    Curr_Vol            =  IniSelVolum; // Set the default current volume if defined.
    Curr_Sli            =  IniSelSlide; // No selected slide.

    // Init related Text Buffer.
    Lab_Text = new Fl_Text_Buffer( 2048, 2048 );

} // DiaViewer_GL::DiaViewer_GL( int x, int y, int w, int h, const char *title ).




void DiaViewer_GL::New_View()
{
    // Temporary limit at two GL windows.
    int i = 0;
    for(i=0;i<GL_WNMAX;i++) if (!Viewer_UI[i]) break;
    if (i<GL_WNMAX) {
        Curr_Sli->CacheSplit();
        DiaWiewer_GL_Init( svargc, svargv, i );
    } else fl_alert( " You cann open a maximum of %d slides's windows.", GL_WNMAX );
} // void DiaViewer_GL::New_View().



void DiaViewer_GL::RkgClearCntx()
// Routine to clear context data for the current directory.
{
        W_sliid->value( slinb_ = 0 );
        W_flmid->value( flmnb_ = 0 );
        W_parid->value( parnb_ = 0 );
        W_chpid->value( chpnb_ = 0 );
} // DiaViewer_GL::RkgEraseNb( int fl ).



void DiaViewer_GL::Viewer_Init()
{
    Win_Init = 0;                       // Do not recall it.

    // Get the DiaViewer Structure (class) adress.
    DiaViewer_UI *Uiw = Viewer_UI[Win_UInb];

    slinb_ = flmnb_ = parnb_ = chpnb_ = profl_ = 0;

    
    // Keep the GL_display address for the new window.

    W_main      =          Uiw->w_main; // Pointer to the FLTK window, ...
    W_fdir      =            Uiw->fdir; // ... to first directory, ...
    W_pdir      =            Uiw->pdir; // ... to previous directory, ...
    W_fsli      =            Uiw->fsli; // ... to first slide, ...
    W_psli      =            Uiw->psli; // ... to previous slide, ...
    W_zoom      =            Uiw->zoom; // ... to the zoom slider, ...
    W_nsli      =            Uiw->nsli; // ... to next slide, ...
    W_lsli      =            Uiw->lsli; // ... to last slide, ...
    W_ndir      =            Uiw->ndir; // ... to next directory, ...
    W_ldir      =            Uiw->ldir; // ... and to last directory.

    W_xscroll   =        Uiw->x_scroll; // Pointer to t X and Y scroll bar, ...
    W_yscroll   =        Uiw->y_scroll;

    W_Class     =         Uiw->w_class; // ... to the classification button, ...
    W_Path      =          Uiw->w_path; // ... to the Slide file path, ...
    W_Text      =          Uiw->w_text; // ... and to the slide comment.
    W_Text->buffer( Lab_Text );         // Attach the text buffer at the text editor.

    W_sliid     =         Uiw->w_sliid; // Slide Number.
    W_flmid     =         Uiw->w_flmid; // Slide Number.
    W_parid     =         Uiw->w_parid; // Slide Number.
    W_chpid     =         Uiw->w_chpid; // Slide Number.
    W_lock      =          Uiw->w_lock; // To can change the lock botton flag.
    W_autom     =         Uiw->w_autom; // To can change the Auto Mode botton flag.
    W_title     =         Uiw->w_title; // Slide anotation Text.

    W_orient    =        Uiw->w_orient; // Orinetation group.
    W_iden      =            Uiw->iden; // Original orientation of slide.
    W_rp90      =            Uiw->rp90; // Slide rotation +90° (anti-clock wise).
    W_rm90      =            Uiw->rm90; // Slide rotation -90° (clock wise).
    W_r180      =            Uiw->r180; // Slide rotation of 180°.
    W_invx      =            Uiw->invx; // Slide mirror mx.
    W_invy      =            Uiw->invy; // Slide mirror my.
    W_invu      =            Uiw->invu; // Slide mirror mu.
    W_invv      =            Uiw->invv; // Slide mirror mv.

    ViewerReady =                    0; // Set to Zero to force the load of first slide.
    ViewRkgEdit =                    0; // By default, no classement allowed.
    ViewRkgAuto =                    1; // By default, Easy mode for classement.
    ViewRkgTitl =                    0; // By default, Display the Directory/Folder name.

} // DiaViewer_GL::Viewer_Init().



void DiaViewer_GL::InitStdScan( VolEntry * vol )
// Method to start the scan of Volume vol with its standard scan program. 
//
{
    int ie;

    if (Curr_Sli)         ChkSliCntxFlags();    // Save Slide context when existing.

    Win_ScDi = 0;

    Curr_Vol = vol;                             // Set this volume as the current one.
    Program = (SlidePrg*)(Curr_Vol->RelPrg());  // Get the Volume scan programe.
    Process = new SlideExec( Curr_Vol,Program );// Initialize the processor.
    ie = Process->Start( 0, SlidePrior );       // Start the program execution ...
                                                // ... that stop on the first slide.
//  printf( " INISTd D\n" ); fflush( stdout );
    if (ie) {
        
        fl_alert( "Scan Program execution error %d.", ie );
        return;
    }

    Curr_Dir = Process->CurrDir();              // Get the directory and the slide.
    Curr_Sli = Process->CurrSli();
///    Curr_Sli->SliRkgClear();

}  // void DiaViewer_GL::InitStdScan( VolEntry * vol ).




void DiaViewer_GL::DiaSetup()
{
    if (SU_Win) Close_Setup();
           else Open_Setup( svargc, svargv );
} // void DiaViewer_GL::DiaSetup().



void DiaViewer_GL::DiaVersion()
{
    if (SU_Version) Close_Version();
           else Open_Version( svargc, svargv );
} // void DiaViewer_GL::DiaVersion().



void DiaViewer_GL::DefSetBlkNbr( int cat )
{
    int nb;

    if (ViewRkgAuto) {
        switch(cat) {
        case 3: nb = W_chpid->value();
            if (nb-1 == chpnb_) {
                chpnb_ = nb;
                W_parid->value( parnb_ = 1 );
                W_flmid->value( flmnb_ = 1 );
                W_sliid->value( slinb_ = 1 );
            }
        break;
        case 2: nb = W_parid->value();
             if (nb-1 == parnb_) {
                parnb_ = nb;
                W_flmid->value( flmnb_ = 1 );
                W_sliid->value( slinb_ = 1 );
             }
        break;
        case 1: nb = W_flmid->value();
            if (nb-1 == flmnb_) {
                flmnb_ = nb;
                W_sliid->value( slinb_ = 1 );
            }
        break;
        }
    }
} // void DiaViewer_GL::DefSetBlkNbr( int cat ).



void DiaViewer_GL::DefSetSliNbr()
// When the slide index is set to a value with all other classement indexies ...
// ... always at 0, then this last ones are set to the values of previous slide values.
//
{
    int snb = W_sliid->value();

    if (snb>0&&ViewRkgAuto) {
        if (!(Curr_Sli->FlgTst( Context_Slinbr ))) {
            // When the slide number is changed from 0 to n ...
            slinb_ = snb;                       // ... the value n is keep.
            if (!(flmnb_||parnb_||chpnb_))      // If all other numbers have the value 0, ...
                flmnb_ = parnb_ = chpnb_ = 1;   // We set film#, paragraph and chapter  to 1.
            // Set the others indexies to there previous slide values.
            if (!W_flmid->value()) W_flmid->value( flmnb_ );
            if (!W_parid->value()) W_parid->value( parnb_ );
            if (!W_chpid->value()) W_chpid->value( chpnb_ ); 
        }
    }
} // void DiaViewer_GL::DefSetSliNbr( int ).



void DiaViewer_GL::CheckEditText()
// Small routine to examine when the current slide anotation
// text changed and set its related flags. 
{
    char * txt = Lab_Text->text(),              // txt: A copy of text in the FLTK editor.
         * old = Curr_Sli->Text();              // old: Pointer to descriptor old slide anotation text.

   if (txt&&txt[0]) {
        Curr_Sli->FlgSet( Context_SText );
        if (old) {                              // New and Old text are ...
            if (strcmp( old, txt )) {           // ... are different ...
                delete[] old;                   // ... we delete the old and ...
                Curr_Sli->Text( txt );          // ... set the new one.
                Curr_Sli->FlgSet( Context_Update );
            } else delete[] txt;                // If the two texts are same, delete the new version.
        } else {                                // The old text does not exist, then ...
            Curr_Sli->Text( txt );              // ... install the new text.
            Curr_Sli->FlgSet( Context_Update );
        }
    } else {                                    // The new text is empty.
        if (txt) delete[] txt;                  // Free a location for an empty text.
        if (old) {
            delete[] old;
            Curr_Sli->Text( NULL );
            Curr_Sli->FlgSet( Context_Update );
            Curr_Sli->FlgClr( Context_SText );
        } else Curr_Sli->FlgClr( Context_SText );
    }   
    txt = old = NULL;
} // void DiaViewer_GL::CheckEditText().



void DiaViewer_GL::CheckEditNumbers()
// Small routine to examine when the current slide parameters
// are modified and set its related flags.
//
{
    // Get the classement values from FLTK spinners
    slinb_  = W_sliid->value(),
    flmnb_  = W_flmid->value(),
    parnb_  = W_parid->value();
    chpnb_  = W_chpid->value();

    if ((slinb_ != Curr_Sli->Numb()) ||
        (flmnb_ != Curr_Sli->Film()) ||
        (parnb_ != Curr_Sli->Para()) ||
        (chpnb_ != Curr_Sli->Chap()) ||
        (profl_ != Curr_Sli->FlgTst( Context_Protect ))) {
        // When some indexies are changed, we set the ...
        // ... Update flag and set the new values.
        Curr_Sli->SliRkgSet( slinb_, flmnb_, parnb_, chpnb_, profl_ );
        Curr_Sli->FlgSet( Context_Update );
        if (slinb_||flmnb_||parnb_||chpnb_||profl_) Curr_Sli->FlgSet( Context_Slinbr );
                                               else Curr_Sli->FlgClr( Context_Slinbr );
    }
} // void DiaViewer_GL::CheckEditNumbers().



void DiaViewer_GL::ChkSliCntxFlags()
// Look at the current slide text and its numbers/flags values
// to determine when some change was occured.
// If yes set the required slide flags before to call
// the SaveSliCntxFlags() slide function to propagate
// it to the volume tree.
//
{
    if (Curr_Sli) {
        CheckEditText();        // Check for anotation text modification.
        CheckEditNumbers();     // Check for parameters modifications.
        // Propagate the possible modification across the volume tree.
        Curr_Sli->SliCntxFlags();
    }
} // void DiaViewer_GL::ChkSliCntxFlags().



void DiaViewer_GL::RkgCntxClear( int fd )
// Clear all classement number of the current slide ,(fd = 0) ...
// ... or all slides of the current directory (fd = 1).
//
{
    slinb_ = flmnb_ = parnb_ = chpnb_ = 0; profl_ = 0;  // Delete memory of previous slide.
    if (fd||!Curr_Sli->FlgTst( Context_ProInb )) {
        ViewRkgAuto = 0;    // Deactivate the Auto Mode to preserv the RkgCntxClear() result(s).
        W_autom->value( 0 );
    }
    if (ViewRkgEdit&&Curr_Sli) {
        Curr_Sli->SliRkgClear();
        if (fd) Curr_Dir->RkgClear();
           else Curr_Sli->RkgClear();
    }
    ShowSlide( 0 );
} // void DiaViewer_GL::RkgCntxClear( int fd ).



void DiaViewer_GL::RkgCntxProtect( int pty, int flg )
{
    int fla;

    if (pty&1) fla  = Context_ProInb;
    if (pty&2) fla |= Context_ProTxt;
    if (pty&4) fla |= Context_ProOri;
    if (pty&8) fla |= Context_LckRank;

    Curr_Sli->SliRkgGet( slinb_, flmnb_, parnb_, chpnb_, profl_ );
    if (flg) LflSet( fla ); else LflClr( fla );
    ChkSliCntxFlags();
    if (Curr_Sli->FlgTst( Context_ProInb )) W_lock->value( 1 );
                                       else W_lock->value( 0 );
    ShowSlide( 0 );
} // void DiaViewer_GL::RkgCntxProtect( int pty, int flg ).



void DiaViewer_GL::RkgCntxZero()
{
    Curr_Sli->SliRkgClear();
    slinb_ = flmnb_ = parnb_ = chpnb_ = 0; profl_ = Curr_Sli->FlgVal();
    RkgCntxProtect( 7, 1 );
} // void DiaViewer_GL::RkgCntxZero().



void DiaViewer_GL::SetDirTitle()
{
    const char * txt = NULL;

    if (Curr_Dir) {
        txt = Curr_Dir->SText();
        if (!txt) txt = def_dtx;
    }
    W_title->label( "in Folder : " );
    W_title->value( txt );  
} // const char * DiaViewer_GL::SetDirTitle().



void DiaViewer_GL::SetVolTitle()
{
    const char * txt = NULL;

    if (Curr_Vol) {
        txt = Curr_Vol->VName();
        if (!txt) txt = def_vtx;
    }
    W_title->label( "in Volume :" );
    W_title->value( txt );  
} // const char * DiaViewer_GL::SetVolTitle().



void DiaViewer_GL::RkgTitleVolDir()
// Select display of Directory name (fd = 0) or volume name (fd = 1).
{
    const char    * txt = NULL;

    if (ViewRkgTitl = 1 - ViewRkgTitl) SetVolTitle();
                                  else SetDirTitle();
} // void DiaViewer_GL::RkgTitleVolDir( int isel ).



void DiaViewer_GL::ActiveProtect()
{
    if (Curr_Sli->FlgTst( Context_ProInb )) {
        W_sliid->deactivate(); W_flmid->deactivate();
        W_parid->deactivate(); W_chpid->deactivate();
        W_lock->value( 1 );
        ViewRkgEdit = 0;
    } else {
        W_sliid->activate(); W_flmid->activate();
        W_parid->activate(); W_chpid->activate();
        W_lock->value( 0 );
        ViewRkgEdit = 1;
    }

    if (Curr_Sli->FlgTst( Context_ProTxt )) W_Text->deactivate();
                                       else W_Text->activate();
    if (Curr_Sli->FlgTst( Context_ProOri )) W_orient->deactivate();
                                       else W_orient->activate();

} // DiaViewer_GL::ActiveProtect().



void DiaViewer_GL::ShowSlide( int init )
//
// **** Routine to make visible the Current Slide ****
//
{
    char    mpath[268];
    int             ie;

//printf( " Try To load file \"%s/%s\n", Process->CurrPath(), Curr_Sli->FName() );

     if (Curr_Sli) {
        snprintf( SliPath, 255, "%s%s", Process->CurrPath(), Curr_Sli->FName() );
        snprintf( mpath, 267, "Path = \"%s\"", SliPath );
        
//printf( "\n path = \"%s\"\n\n", fpath );

        W_Path->value( mpath );             // Edit the window file reference.

        if (ViewRkgTitl) SetVolTitle();     // Set the related title (Volume or Folder/directory).
                    else SetDirTitle();

        if (Curr_Map = Curr_Sli->Get( SliPath, Win_UInb ))
        {   // We have loaded the Slide in the Image Cache.
            i_widt  = Curr_Sli->W();
            i_high  = Curr_Sli->H();
            i_pixs  = Curr_Sli->P();

            i_shww  = i_hwid = i_widt/2;
            i_shhh  = i_hhig = i_high/2;
            i_cenx  = i_ceny = 0.0;          // Set the default center of display in the image.
            img_x1  = -(img_x2 = i_hwid);    // Set the image (PixMap) limits.
            img_y1  = -(img_y2 = i_hhig);
            FirstView = 1;

            // Display all context values.
//printf( " msv %d, mfn %d, mcn %d, mvn %d\n", slinb_, flmnb_, parnb_, chpnb_ );

            Curr_Sli->SliRkgGet( slinb_, flmnb_, parnb_, chpnb_, profl_ );
            // When the ranking data are locked (protected) ...
            ActiveProtect();
            if (ViewRkgAuto&&!Curr_Sli->FlgTst( Context_ProInb )&&
                Process->PrvRkgTest()&&!(slinb_||flmnb_||parnb_||chpnb_)) {
                Process->PrvRkgGet( slinb_, flmnb_, parnb_, chpnb_ );
                if (slinb_ < slin_max) slinb_++;
            }
//printf( " sn %d, fn %d, cn %d, vn %d, flg %d\n", slinb_, flmnb_, parnb_, chpnb_, Curr_Sli->FlgVal() );
            W_sliid->value( slinb_ );
            W_flmid->value( flmnb_ );
            W_parid->value( parnb_ );
            W_chpid->value( chpnb_ );

            Lab_Text->text( Curr_Sli->Text() );

            
            // To display the slide comment (in future).
            if (!init ) redraw();
        } else {
            ie = Curr_Sli->E();
            fl_alert( "DiaViewer : Cannot load the slide \"%s\"\n with error %d .",
                      Curr_Sli->FName(), -ie );
        }
    }
    else
        W_Path->value( "  << NO Image ! >>" );


} // void DiaViewer_GL::ShowSlide( int init ).



int  DiaViewer_GL::RkgEditEnDi()
// Lock or unlock the Ranking data edition (when pôssible). 
{
    if (ViewRkgEdit) {
        ChkSliCntxFlags();      // Save the Ranking context.
        ViewRkgEdit = 0;
        W_sliid->deactivate();
        W_flmid->deactivate();
        W_parid->deactivate();
        W_chpid->deactivate();
        return 0;
    } else {
        DiaViewer_GL * WGl;
        int         nv = 0;

        for (int i=0; i<GL_WNMAX; i++) {
            if (Viewer_UI[i])
                if (WGl = Viewer_UI[i]->w_image) if (WGl->ViewRkgEdit) {
                    fl_message( "Not allowed operation:\n The DiaViewer windows #%d is already in classement mode.",
                                i+1 );
                    return 0;
                }
        }
        if (nv) {
            fl_message( "Not allowed operation:\n The DiaViewer windows #%d is in classement mode.", nv );
            return 0;
        }
        ViewRkgEdit = 1;
        W_sliid->activate();
        W_flmid->activate();
        W_parid->activate();
        W_chpid->activate();
    }
    return ViewRkgEdit;
} // DiaViewer_GL::RkgEditEnDi( int fl ).



void DiaViewer_GL::NormalEnd( int flg = 0 )
{
    int wnb = 0;

    for(int i=0; i<GL_WNMAX; i++) if (Viewer_UI[i]) wnb++;

    if (wnb>1&&(flg==0)) { // When other views are opened, we just close the present one.
        Viewer_UI[Win_UInb] = NULL;
        ChkSliCntxFlags();
        Curr_Sli->CacheMerge();
        delete W_main;
    } else {
        for(int i=0; i<GL_WNMAX; i++)
            if (Viewer_UI[i]) {
                DiaViewer_GL * WGl = Viewer_UI[i]->w_image;
                WGl->ChkSliCntxFlags();
            }
        SaveAllApplContext( Curr_Vol );
        if (CloseAllVol()) exit( 0 );
    }
} // void DiaViewer_GL::NormalEnd( int flg = 0 ).



void DiaViewer_GL::SetSize( int ww, int hh )
// Force the size (arguments in GL space).
{
    
} // void DiaViewer_GL::SetSize( int ww, int hh ).



void DiaViewer_GL::VolClose()
// To close and free all structures of a Slide volume.
{
#ifdef GL_DEBUG
    static const
        char dnam[]="<noname>";
#endif
    int             idx =    0,
                 jdx, kdx, ldx;
    VolEntry            * rvol,
                        * cvol;
    DiaViewer_UI        *    w;
    
    if (!OpenVolNb) return;     // Nothing to do when no opened volume.

    idx = SlideVolSelect(  "Select the Volume to close in this list", cvol );

//printf( " Idx = %d, path=%s\n", idx, cvol->VRoot()->FName() ); fflush( stdout );
    
    if (!idx||!cvol) return;

/// cvol->WriteVolConstexts();  // Update Volume context when required.

    rvol = NULL;
    jdx = SetUpQuLocate( VolQueue, -1 );        // Init a Volume queue scan.
    kdx = ldx = 0;
//printf( " AA\n" ); fflush( stdout );
    while (jdx&&(!ldx)) { // Loop to search neibouring opened volume.
        if (jdx<idx&&kdx<jdx) kdx = jdx;        // Keep a previous volume index.
        if (jdx>idx) ldx = jdx;                 // If a volume exist after to close one set its index.
        jdx = SetUpQuLocate( VolQueue, 0 );     // Skip to next volume.
    }
//printf( " AB\n" ); fflush( stdout );

    if (!kdx&&ldx) kdx = ldx;                   // If no previous and next was found select it ...
    if (kdx) { // We have an other volume to select.
        jdx = SetUpQuLocate( VolQueue, kdx );   // ...  and else we can select the next one.
        rvol = (VolEntry*)SetUpQuGetData( VolQueue );
    }
//printf( " AC\n" ); fflush( stdout );

    // Loop to close all slides's window using the volume to close.
    for(int i =0; i<GL_WNMAX; i++) {            // Loop on all opened slides view.
        if (w = Viewer_UI[i]) {                 // Get the window of this view.
            if (w->w_image->Curr_Vol == cvol) { // When it is a view of the volume to close ...
// printf( " CLOSE A %d\n", i ); fflush( stdout );
                if (ViewRkgEdit) w->w_image->ChkSliCntxFlags(); // We perform a context save.
                Curr_Vol = rvol;
                if (rvol) {
                    w->w_image->InitStdScan( rvol );
                    w->w_image->ShowSlide( 0 );
                } else Curr_Vol = NULL;
// printf( " CLOSE B\n"); fflush( stdout );
            }
        }
    }
// printf( " AD\n" ); fflush( stdout );

    CloseSlideVolume( cvol );        // Free the volume location
// printf( " AE\n" ); fflush( stdout );
} // void DiaViewer_GL::VolClose().



int DiaViewer_GL::CloseAllVol()
// To close all opened files and directories.
// To use only for the end of DiaViewer Execution.
{
    int idx = SetUpQuLocate( VolQueue, -1 );    // Init a scan of the Volume list.
    while (idx) {                               // Loop on all open volume.
        Curr_Vol = (VolEntry*)(SetUpQuGetData( VolQueue ));
        idx = SetUpQuLocate( VolQueue, 0 );
    }
    Curr_Sli = NULL;
    Curr_Dir = NULL;
    Curr_Vol = NULL;
    return 1;
} // int DiaViewer_GL::CloseAllVol().



void DiaViewer_GL::VolOpen( int dflg ) {
// Open a directory and scan it.
// When (dflg) the user interaction result is an image name, ...
// ... else it is a directory name.
// A New Slide volume is created.
//
    int ie, ii, jj, kk, ll;
    char                    ch;
    char      * fnam,   * fdir;
    const  char        * fpath;

    if (OpenVolPath) fnam = strdup( OpenVolPath );
                else SetUpSolveLogPath( UserDir, fnam, ll );
        
    fpath = DLG_DirChooser( "DiaViewer: Selecte Slide Directory", fnam );
    if (fpath) {
        delete[] OpenVolPath;
        OpenVolPath = strdup( fpath );
        delete[] fnam;
    } else {
        OpenVolPath = fnam;
        return;
    }

    ie = OpenSlideVolume( fpath, dflg );
    if (ie) return;

    ChkSliCntxFlags();          // Save current volume ranking context.
    Curr_Vol = IniSelVolum;
    InitStdScan( Curr_Vol );
    ShowSlide( 0 );
}  // void DiaViewer_GL::VolOpen( int dflg ).



void DiaViewer_GL::VolSelect()
// To change of slide volume.
{
    static const
        char dnam[]="<noname>";

    int             idx =    0;
    VolEntry      * vol = NULL;
    const char          * vnam;
    
    if (OpenVolNb < 2) return;  // Nothing to do.

    idx = SlideVolSelect(  "Select a Volume in this list", vol );

    if (idx>0) {
        vnam = vol->VName();
        if (!vnam) vnam = dnam;

#ifdef GL_DEBUG
    printf( " Volume # %3d : %s Path=\"%s\"\n", idx, vnam, vol->VRoot()->FName() );
#endif

        ChkSliCntxFlags();
        Curr_Vol->WriteVolContext();
        Curr_Vol = vol;
        InitStdScan( vol );
        ShowSlide( 0 );
    }
        
} // void DiaViewer_GL::VolSelect().



void DiaViewer_GL::DiaSearch()
{
}



void DiaViewer_GL::SaveContext()
{
    ChkSliCntxFlags();
    Curr_Vol->WriteVolContext();
}



void DiaViewer_GL::VolCreate()
// To create a new (and empty) slide volume.
{
    
} // void DiaViewer_GL::VolCreate().



void DiaViewer_GL::DirScan( int chs )
// Change of slide directory in the same volume.
{
    Win_ScDi = 0;
    if (Curr_Sli) ChkSliCntxFlags();
    if (!Process->Continue( chs, SlidePrior )) {
        Curr_Dir = Process->CurrDir();  // Get the directory and the slide.
        Curr_Sli = Process->CurrSli();
        RkgClearCntx();
        ShowSlide( 0 );
    }
} // void DiaViewer_GL::DirScan( int chs ).



void DiaViewer_GL::SliScan( int id )
// To manage the scan inside a slide directory.
// id values and effect.
//  1 to skip to the first slide of current directory,
//  0 to skip to the next slide,
// -1 to skip to the previous slide,
// -2 to skip to the last slide of current directory.
//
{
    switch (id) {
        case -2:
        case  1: Win_ScDi =  0; break;
        case -1: Win_ScDi = -1; break;
        case  0: Win_ScDi =  1; break;
        default: ;
    }
    if (Curr_Sli) ChkSliCntxFlags();
    if (Process->SlideScan( id )==0)
    { // We operate in an existing directory.
        Curr_Sli = Process->CurrSli();
        ShowSlide( 0 );
    } else {
        if (ScanNoLim&&(id==0||id==-1)) DirScan( id );
    }
} // void DiaViewer_GL::SliScan( int id ).



void DiaViewer_GL::DirRestart( int chs )
// Restart to first(chs<0) or last(chs>=0) slide in the volume..
{
    int id = (chs<0)? 1 : -1;

    
    if (Curr_Sli) ChkSliCntxFlags();
    Process->Abort();

    Win_ScDi = 0;
    if (!Process->Start( id, SlidePrior )) {
        Curr_Dir = Process->CurrDir();  // Get the directory and the slide.
        Curr_Sli = Process->CurrSli();
        RkgClearCntx();
        ShowSlide( 0 );
    }
} // void DiaViewer_GL::DirRestart( int chs ).



void DiaViewer_GL::SliTransf( Image_REF::Image_Trf trf )
// Modify by rotation or mirror the selected Slide.
{
    if (Curr_Sli) ChkSliCntxFlags();
    int t = Curr_Sli->Transform( trf, SliPath, Win_UInb );
    switch (t) {
        case 0: Curr_Sli->FlgClr( Context_Orient );
        break;
        case 1: Curr_Sli->FlgSet( Context_Orient );
        break;
        case 2: { Curr_Sli->FlgSet( Context_Update ); Curr_Sli->FlgClr( Context_Orient ); }
        break;
        case 3: Curr_Sli->FlgSet( Context_Orient|Context_Update );
        break;
    }
    ShowSlide( 0 );
} // void DiaViewer_GL::SliTransf( Image_REF::Image_Trf trf ).



void DiaViewer_GL::XScroll()
{
    GLint vl, sz;

    vl = W_xscroll->value();

    if (vl >= 0 && vl < i_widt - i_npix) {
//      printf( " X Scroll val = %d\n", vl  );
        X_shift += vl - i_spix;
        redraw();
    }
} // void DiaViewer_GL::XScroll().



void DiaViewer_GL::YScroll()
{
    GLint vl, sz;

    sz = i_high - i_nrow;
    vl = sz - W_yscroll->value();
    if (vl >= 0 && vl < sz) {
//      printf( " Y Scroll val = %d\n", vl );
        Y_shift += vl - i_srow;
        redraw();
    }
} // void DiaViewer_GL::YScroll().



void DiaViewer_GL::SetScale( float val )
{
//  printf( " Scale Value = %8.4f\n", val );
    i_shww = w_hwid/val; i_shhh = w_hhig/val;
    r_scale = val;
    SetScaVal = 1;
    redraw();
} // DiaViewer_GL::SetScale( float val ).


void DiaViewer_GL::UnZoom( int fl )
{
    i_shww = i_hwid;
    i_shhh = i_hhig;
    i_pixs = i_pixs;
    i_cenx = i_ceny = 0;
    FirstView = 1;
    redraw();
} // void DiaViewer_GL::UnZoom( int fl ).



void DiaViewer_GL::Classify()
{
    Rkg_Setup( svargc, svargv, Curr_Vol );
    
} // void DiaViewer_GL::Classify().



void DiaViewer_GL::SliPrint()
// Print the current View of Diapositive.
{
    Fl_Printer printer;
    int width, height;

    ViewToPrint = this;  // To find the related slide to print.

    if (printer.start_job(1)) return;
    printer.start_page();
    printer.printable_rect( &width, &height );
    float fw = w_widt / float( width );
    float fh = w_high / float( height );
    if (fh > fw) fw = fh;
    printer.scale( 1/fw );
    printer.print_widget( this );
    printer.end_page();
    printer.end_job();
} // void DiaViewer_GL::SliPrint().




//
// Private methods.
//

void DiaViewer_GL::MouseDown( int x, int y, int button )
{
    if (!MouseStat) {                   // When the mouse is active without button already pushed.
        zre_x1 = x, zre_y1 = y;
        zre_x2 = x, zre_y2 = y;
        fl_cursor( FL_CURSOR_CROSS );
        UsedButton = button;
        MouseStat = 1;
    }
} // void DiaViewer_GL::MouseDown( int x, int y, int button ).



void DiaViewer_GL::MouseDrag( int x, int y, int button )
{;
    if (UsedButton!=button) {
         MouseStat = 0; return;
    }
    if ((x!=zre_x2)||(y!=zre_y2)) {
        zre_x2 = x; zre_y2 = y;
        redraw();
    }
} // void DiaViewer_GL::MouseDrag( int x, int y, int button ).



void DiaViewer_GL::MouseUp( int x, int y, int button )
{
    GLfloat zw, zh;

    if (UsedButton!=button) {
         MouseStat = 0; return;
    }
    zre_x2 = x; zre_y2 = y;
    fl_cursor( FL_CURSOR_DEFAULT );
    MouseStat = 0;

    zw = 0.5*abs( zre_x2 - zre_x1 ); zh = 0.5*abs( zre_y2 - zre_y1 );
    if ((zw > 50.0)&&(zh > 50.0)) {
        // Increase one rectangle size to fit with window proportions.
        if (zh*w_hwid > w_hhig*zw) zw = (zh*w_hwid)/w_hhig;
                              else zh = (zw*w_hhig)/w_hwid;
        // Get the showed view center translation vector (or new center coord.) ...
        // ... in the image coordinates.
        i_zcex = round( 0.5*(zre_x1 + zre_x2) ) /* + i_cenx */;
        i_zcey = round( 0.5*(zre_y1 + zre_y2) ) /* + i_ceny */;
        i_zshw = round( zw ); i_zshh = round( zh );

#ifdef GL_DEBUG
        printf( " SCALE = %f\n", r_scale );
        printf( " Z rect : x1=%d, y1=%d, x2=%d, y2=%d\n", zre_x1, zre_y1, zre_x2, zre_y2 );
        printf( " Center at (%d, %d)\n", i_zcex, i_zcey );
#endif

        ZoomStatus = 1;
    }
    redraw();
} // void DiaViewer_GL::MouseUp( int x, int y, int button ).



void DiaViewer_GL::XYinImage( int &x, int &y )
// Get the pointed coordinate in the image pixels space.
{
    x = round( (Fl::event_x() - w_hwid)/r_scale ) + i_cenx;
    y = round( (w_hhig - Fl::event_y())/r_scale ) + i_ceny;
} // void DiaViewer_GL::XYinImage( int &x, int &y ).



int  DiaViewer_GL::handle( int evn )
// Call to manage the mouse or keybord events.
//
{
    int     mx, my;
    int         bt;

    bt = Fl::event_button();
    switch (evn) {
        case FL_PUSH:
            XYinImage( mx, my );
            MouseDown( mx, my, bt );
            return 1;
        break;
          
        case FL_DRAG:
            XYinImage( mx, my );
            MouseDrag( mx, my, bt );
            return 1; redraw();

        break;
            
        case FL_RELEASE:
            XYinImage( mx, my );
            MouseUp( mx, my, bt );
            return 1;

        break;
        
//      case FL_MOUSEWHEEL:
//           
//      break;
        
        default: ;
    }
    return Fl_Gl_Window::handle( evn );
}  // int  DiaViewer_GL::handle( int evn ).




void DiaViewer_GL::draw()
// called to re-draw the GL window.
//
{
    GLint    ierr,     map_size,
            whwid,        whhig, // Half display size in image pixels.
            map_x,        map_y; // PixelMap origine on Display.
    GLenum               maptyp;
    GLfloat  rx_scale, ry_scale;

    if (Win_Init) Viewer_Init();

#ifndef MESA
    glDrawBuffer(GL_FRONT_AND_BACK);
#endif // !MESA

    if (!valid()) {             // GL Init is done.
        w_widt  =          w(); w_high  =          h();
        w_hwid  =     w_widt/2; w_hhig  =     w_high/2;
        // Set the display pixel origine on the GL Window center in a 3D space.
//      glOrtho( (GLfloat)-w_hwid, (GLfloat)w_hwid, (GLfloat)-whhig, (GLfloat)w_hhig, -20.0, 20.0 );
        valid( 1 );
    }

    if (!ViewReady) { 
        ViewReady  = 1;
        if (Curr_Vol) InitStdScan( Curr_Vol );
        if (Curr_Sli&&Curr_Dir) ShowSlide( 1 );
    }

    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    if (Curr_Map) {
        
//      Already done by ShowSlide().
//      i_shww = i_hwid = (i_widt = Curr_Sli->W())/2;
//      i_shhh = i_hhig = (i_high = Curr_Sli->H())/2;
//      ii_pxs = Curr_Sli->P();

#ifdef GL_DEBUG
        if (!MouseStat) {
            printf( " Half Window sizes (screen pixels) : %d, %d\n", w_hwid, w_hhig );
            printf( " Half image sizes  (image  pixels) : %d, %d\n", i_hwid, i_hhig );
        }
#endif

        if (ZoomStatus) {
            // Test validity of the zoom.
            if (i_zcex-i_zshw < i_hwid && i_zcex+i_zshw > -i_hwid &&
                i_zcey-i_zshh < i_hhig && i_zcey+i_zshh > -i_hhig) {
                // Update the center of showed rectangle in image coordinates.

#ifdef GL_DEBUG             
        printf( " Zoom Half showed sizes (image  pixels) : %d, %d\n", i_zshw, i_zshh );
        printf( " Zoom Center of showed zoom (image  pixels) : %d, %d\n", i_zcex, i_zcey );
#endif

                FirstView = 0;
                if ((GLfloat)w_hwid/i_zshw <= 5.0 || (GLfloat)w_hhig/i_zshh <= 5.0)
                { // Limit the scale at 5.0 (5 Screen_Pixel/Image_Pixel)
                    i_cenx = i_zcex; i_ceny = i_zcey;
                    i_shww = i_zshw; i_shhh = i_zshh;
                }
            }
            ZoomStatus = 0;
        }

        if (X_shift) {
            i_cenx += X_shift;
            X_shift = 0;
        }

        if (Y_shift) {
            i_ceny +=  Y_shift;
            Y_shift = 0;
        }

#ifdef GL_DEBUG
        if (!MouseStat) {
            printf( " Half showed sizes (image  pixels) : %d, %d\n", i_shww, i_shhh );
        }
#endif

        // Compute the display scale (in Display_Pixels/Image_Pixels).
        if (!SetScaVal) {
            rx_scale = (GLfloat)w_hwid/i_shww; ry_scale = (GLfloat)w_hhig/i_shhh;
            r_scale = (rx_scale < ry_scale)? rx_scale : ry_scale;
            if (FirstView&&(r_scale > 1.0)) r_scale = 1.0;
        } else SetScaVal = 0;

        W_zoom->value( r_scale );
        whwid = w_hwid/r_scale; whhig = w_hhig/r_scale;

#ifdef GL_DEBUG        
        if (!MouseStat) {               
            printf( " scale = %10.6f\n", r_scale );
            printf( " Half Display sizes (Image pixels) : %d, %d\n", whwid, whhig );
        }
#endif

        // Get the future visible image part rectangle limits.
        cur_x1 = i_cenx -  whwid; cur_y1 = i_ceny -  whhig;
        cur_x2 = i_cenx +  whwid; cur_y2 = i_ceny +  whhig;

        // Compute the pixel and row to skip when required.
        if (img_x1 < cur_x1) { i_spix = cur_x1 - img_x1; map_x = cur_x1; }
                        else { i_spix = 0; map_x = img_x1;}
        i_npix = ((img_x2 > cur_x2)? cur_x2 : img_x2) - map_x;

        if (img_y1 < cur_y1) { i_srow = cur_y1 - img_y1; map_y = cur_y1; }
                        else { i_srow = 0; map_y = img_y1; }
        i_nrow = ((img_y2 > cur_y2)? cur_y2 : img_y2) - map_y;

        W_xscroll->value( i_spix, i_npix, 0, i_widt );
        W_yscroll->value( i_high -i_srow - i_nrow, i_nrow, 0, i_high );

#ifdef GL_DEBUG
        if (!MouseStat) {               
            printf( " *** Image space Rectangle : %d, %d, %d, %d\n", cur_x1, cur_y1, cur_x2, cur_y2 );
            printf( " *** Scale = %8.5f, MapSizes = (%d, %d)\n", r_scale, i_npix, i_nrow );
            printf( " *** Skip Pixels,Rows (%d, %d), MapOrg. = (%d, %d)\n", i_spix, i_srow, map_x, map_y );
        }
#endif

        // Define the showed image space.
        glMatrixMode( GL_PROJECTION );
        glLoadIdentity();
        glViewport( 0, 0, w_widt, w_high);  // Set the GL ViewPort.
        glOrtho( (GLfloat)cur_x1, (GLfloat)cur_x2,
                 (GLfloat)cur_y1, (GLfloat)cur_y2,
                            -1.0,             1.0
               );
        glEnable( GL_DEPTH );
        glMatrixMode( GL_MODELVIEW );
        glLoadIdentity();
    
//      glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
//  printf( " draw() U %d\n", i_pixs ); fflush( stdout );

        switch (i_pixs) {
            case 1: maptyp = GL_LUMINANCE; break;
            case 3: maptyp =       GL_RGB; break;
            case 4: maptyp =      GL_RGBA; break;
  
            default:
                fl_alert( "DiaViewer: Bad type of imag: %d byte/pixel.", i_pixs );
                maptyp = -1;
                printf( " Bad type of image\n" ); fflush( stdout );
//              exit( 2 );
        }

//  printf( " draw() V %d\n", maptyp ); fflush( stdout );
        if (maptyp>0) {
//  printf( "-----> i_widt=%d, i_spix=%d, i_srow=%d, at (%d,%d)\n", i_widt,i_spix,i_srow, map_x, map_y);
//  fflush( stdout );
            glPixelStorei( GL_UNPACK_ROW_LENGTH,  i_widt );
            glPixelStorei( GL_UNPACK_SKIP_PIXELS,  i_spix );
            glPixelStorei( GL_UNPACK_SKIP_ROWS,    i_srow );
            glPixelStorei( GL_UNPACK_ALIGNMENT,        1 );
            glRasterPos3f( map_x+0.0001, map_y+0.0001, 0.0 );
//  GLboolean valid; glGetBooleanv( GL_CURRENT_RASTER_POSITION_VALID, &valid );
//  printf( " PIxmap Position valid = %d\n", valid );
            glPixelZoom( r_scale, r_scale );
            glDrawPixels(  i_npix, i_nrow, maptyp, GL_UNSIGNED_BYTE, (GLvoid*)Curr_Map );
        }
        
//  printf( " draw() W\n" ); fflush( stdout );
       if (MouseStat) {
            glLineWidth( 2.0 );
            glColor3f( 0.5, 0.5, 0.5 );
            glBegin( GL_LINE_LOOP );
            glVertex3f( zre_x1, zre_y1, -1.0 );
            glVertex3f( zre_x2, zre_y1, -1.0 );
            glVertex3f( zre_x2, zre_y2, -1.0 );
            glVertex3f( zre_x1, zre_y2, -1.0 );
            glEnd();
        }        
    }
//  printf( " draw() X\n" ); fflush( stdout );

#ifndef MESA
    glDrawBuffer(GL_BACK);
#endif // !MESA

//  printf( " draw() Z\n" ); fflush( stdout );
      
} // void DiaViewer_GL::draw().



//
// end of <PW-Id>.
//
