//
// <PW-Id>
//

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// Copyright 2018-2020 by Pierre Wolfers Meylan France                       //
//                                                                           //
// This library is free software. Distribution and use rights are outlined   //
// in the file "COPYING" which should have been included with this file.     //
// If this file is missing or damaged, see the license at:                   //
//                                                                           //
//    <To be define when ready>                                              //
//                                                                           //
//   This license described in this file overrides all other licenses that   //
//   might be specified in other files for this library.                     //
//                                                                           //
//   This library is free software; you can redistribute it  and/or modify   //
//   it under the terms of the GNU Lesser General Public License as publi-   //
//   shed by  the  Free Software  Foundation;  either  version 2.1  of the   //
//   License, or (at your option) any later version.                         //
//                                                                           //
//   This library  is  distributed in  the  hope that  it will  be useful,   //
//   but   WITHOUT  ANY  WARRANTY;  without even  the  implied warranty of   //
//   MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the GNU   //
//   Library General Public License for more details.                        //
//                                                                           //
//   You should have  received  a copy of  the  GNU Lesser General  Public   //
//   License  along with this library  (see COPYING.LIB); if not, write to   //
//   the Free Software Foundation :                                          //
//                        Inc., 675 Mass Ave, Cambridge, MA 02139, USA.      //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



//
// P.Wolfers Software
//

//
// Module to scan slides in a directory
//

#include <FL/Fl.H>
#include <FL/fl_ask.H>

#include <string.h>

#include "SetUpMan.h"

#include "DiaViewer_SRV.h"
#include "DiaViewer_DCM.h"
#include "DiaViewer_GL.h"





//
// Main SetUp file variables.
//

int win_dx = 860, win_dy = 715; // Default Window size.


SetUpQueue  VolQueue =    NULL; // Queue of opened (or to open) slides Volumes.


SetUpDynTab
    VolToOpen    = { 0, NULL }, // To keep the volume list to open.
    WinTable     = { 0, NULL }, // To keep all windows (to create at start time) data.
    // The next tables should be replaced by the table WinTable.
    WinVolume    = { 0, NULL }, // To keep the volume windows links (temporary for compatibility).
    WinSelSli    = { 0, NULL }; // To keep the position of displayed slides (temporary for compatibility).

ObjTable          OpenVolTable; // The table of all opened slides volumes.


int        ScanNoLim =       1, // Scan no limited to sub-directories limits (default yes).
          SlidePrior =       0, // Flag for scan priority for directories ...
                                // ... (for 0) or for slides if not 0.
          NativeOFil =       0; // Flag to use native openfile and directory.



const char    UserEnv[] =           USERENV;
const char    UserDir[] =           USERDIR;
const char DiaEnvFile[] = ".DiaViewer.conf";




//
// Main SetUp file definitions
//

static
    const SetUpDsc  Scr_Sizes[] = {
        { SetUpInt,   1,    AD( win_dx ) },
        { SetUpInt,   1,    AD( win_dy ) },
        { SetUpEof,   0,               0 }
    };

static
    const SetUpDsc Rank_SliFrm[] = {
        { SetUpStr,   1,  AD( slid_hde ) },
        { SetUpInt,   1,  AD( film_dig ) },
        { SetUpStr,   1,  AD( slid_sep ) },
        { SetUpInt,   1,  AD( slid_dig ) },
        { SetUpEof,   0,               0 }
    };

static
    const SetUpDsc Rank_DirFrm[] = {
        { SetUpStr,   1,  AD( fold_hde ) },
        { SetUpInt,   1,  AD( fold_dig ) },
        { SetUpStr,   1,  AD( fold_tra ) },
        { SetUpEof,   0,               0 }
    };

static
    const SetUpDsc Rank_Sizes[] = {
        { SetUpInt,   1,  AD( dirslisz ) },
        { SetUpInt,   1,  AD( dirdirsz ) },
        { SetUpEof,   0,               0 }
    };

static
    const SetUpDsc SlNb_Maxs[] = {
        { SetUpInt,   1,  AD( flmn_max ) },
        { SetUpInt,   1,  AD( slin_max ) },
        { SetUpEof,   0,               0 }
    };

// List od defined Setup file identifier :
// In this list, when the field adress (vaddr) is NULL the object ...
// ... is ignored in input and never write in output.
static
    const SetUpIde MainSetUpFile[] = {
        {   "SCREEN_SIZES", { SetUpRec,    2,    AD( Scr_Sizes ) } },
        {   "VOLUME_LISTE", { SetUpQue,    0,     AD( VolQueue ) } },
        {  "SELECT_VOLUME", { SetUpInt,    1,               NULL } },
        { "SLIDE_PRIORITY", { SetUpInt,    1,   AD( SlidePrior ) } },
        {    "SCAN_NO_LIM", { SetUpInt,    1,    AD( ScanNoLim ) } },
        {  "PRINT_FROM_GL", { SetUpInt,    1,    AD( PrtFromGL ) } },
        {    "NATIVE_OPEN", { SetUpInt,    1,   AD( NativeOFil ) } },
        {     "CACHE_SIZE", { SetUpInt,    1, AD( SliCacheSize ) } },
        {  "INT_IDENT_MAP", { SetUpInt,    2,      AD( IIdeFld ) } },
        { "SLIDES_NUMBERS", { SetUpRec,    2,    AD( SlNb_Maxs ) } },
        { "SLIDE_RANK_FRM", { SetUpRec,    4,  AD( Rank_SliFrm ) } },
        {   "DIR_RANK_FRM", { SetUpRec,    3,  AD( Rank_DirFrm ) } },
        {    "SLIDIR_RANK", { SetUpRec,    2,   AD( Rank_Sizes ) } },
        {   "OPENVOL_PATH", { SetUpStr,    1,  AD( OpenVolPath ) } },
        {    "TARGET_PATH", { SetUpStr,    1,     AD( tvol_pth ) } },
        {    "TARGET_NAME", { SetUpStr,    1,     AD( tvol_nam ) } },
        { "SAVE_CNTX_PERI", { SetUpInt,    1,               NULL } },
        { "VOLUMES_SELECT", { SetUpDSt,    1,    AD( VolToOpen ) } },
        {  "WINDOWS_TABLE", { SetUpDIn,    1,     AD( WinTable ) } },
        { "WINDOWS_VOLUME", { SetUpDIn,    1,               NULL } }, // Temporary.
        { "WINSLID_SELECT", { SetUpDIn,    1,               NULL } }, // Temporary.
        {                0, { SetUpEof,    0,                  0 } }
    };







void ErrorMessage( int ie, const char *msg, const char *path )
// Graphic Error message routine.
//
{
    
}




void SaveAllApplContext( VolEntry * cvol )
// Should be called by exit processing.
{
    SetUpInsPath( UserEnv );                            // Select the File SetUp Path.
    SetUpWrite( DiaEnvFile,"DiaViewer",MainSetUpFile ); // Write or Update the SetUp Base File.
} // SaveAllApplContext.



int main( int argc, char ** argv )
{
    int p2 = 1;

    printf( "User Env = \"%s\"\n\n", UserEnv );
    VolQueue = SetUpCreateQueue( SetUpStr );    // Init the Queue of Volumes to Open.
    SetUpInsPath( UserEnv );                    // Select the File SetUp Path.
    SetUpRead( DiaEnvFile, MainSetUpFile );     // Read the SetUp Base File.
    
    OpenVolumeQuList();                         // Open all volume in VolQueue.

    if (!WinTable.size) {                       // When we have no config file (or too old) ...
        WinTable.size = 4;                      // ... we create it for one window.
        WinTable.tab.i = new int[4];
        for(int i = 0; i < 4; i++) WinTable.tab.i[i] = 0;
    }

    DiaViewer_GL::Init( argc, argv, 0, 1 );     // Init the windowing system (display 0).

/*  for(int i = 1; i < WinSelect.size; i++) {   // Create each window as setup specified.
        SliEntry::CacheSplit( i );
        if (WinSelect.itab[i])
            DiaViewer_GL_Init( argc, argv, 1 );
    }
*/
    return Fl::run();
}



//
// end of <PW-Id>.
//
