// <PW-Id>
//

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// Copyright 2018-2020 by Pierre Wolfers Meylan France                       //
//                                                                           //
// This library is free software. Distribution and use rights are outlined   //
// in the file "COPYING" which should have been included with this file.     //
// If this file is missing or damaged, see the license at:                   //
//                                                                           //
//    <To be define when ready>                                              //
//                                                                           //
//   This license described in this file overrides all other licenses that   //
//   might be specified in other files for this library.                     //
//                                                                           //
//   This library is free software; you can redistribute it  and/or modify   //
//   it under the terms of the GNU Lesser General Public License as publi-   //
//   shed by  the  Free Software  Foundation;  either  version 2.1  of the   //
//   License, or (at your option) any later version.                         //
//                                                                           //
//   This library  is  distributed in  the  hope that  it will  be useful,   //
//   but   WITHOUT  ANY  WARRANTY;  without even  the  implied warranty of   //
//   MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the GNU   //
//   Library General Public License for more details.                        //
//                                                                           //
//   You should have  received  a copy of  the  GNU Lesser General  Public   //
//   License  along with this library  (see COPYING.LIB); if not, write to   //
//   the Free Software Foundation :                                          //
//                        Inc., 675 Mass Ave, Cambridge, MA 02139, USA.      //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



//
// P.Wolfers DiaViewer Software
//

//
// Environment of Service routines/object.
//


#ifndef DIAVIEWER_SRV
#define DIAVIEWER_SRV

#include "ErrMsgMan.h"
#include "SetUpMan.h"



//
// Some global definitions.
//

//
// Define the DiaViewer Version for possible future changes.
//
#define DIAVIEWER_VERSION                     1
#define DIAVIEWER_SUB_VERSION                 1
#define DIAVIEWER_RELEASE                 "A01"
#define DIAVIEWER_VERFORMAT "DVW %3d.%3.3u-%3s"

// The next definition must be in agreement with the previous one and ...
// ... the fixed format "DVW %3d.%3.3u-%3s"
#define DIAVIEWER_STRVERS     "DVW   1.001-A01"



#ifndef DIRSEP
#   if defined(WIN32) || defined(__EMX__) && !defined(__CYGWIN__)
#       define DIRSEP '\\'
#   else
#       define DIRSEP '/'
#       define O_BINARY 0
#   endif
#endif

#if !(defined(WIN32) || defined(__EMX__) && !defined(__CYGWIN__))
# define O_BINARY 0
#endif


// User environment file space (depending of operating system).

#if defined( WINHOME ) || defined( WIN32 ) 
# define USERENV "LOCALAPPDATA:"
# define USERDIR "USERPROFILE:"
#else
# define USERENV "HOME:"
# define USERDIR "HOME:"
#endif



#define PATH_ALLOC_STEP 64



//
// Define some common types.
//
typedef unsigned  char   UChar; // Our insigned character or unsigned 8 bits.
typedef unsigned short  UInt16; // Our Insigned 16 bits.
typedef unsigned   int  UInt32; // Our Insigned 32 bits.
typedef            char  SChar; // Our signed character or signed 8 bits.
typedef          short  SInt16; // Our Signed 16 bits.
typedef            int  SInt32; // Our Signed 32 bits.
typedef         void *  ObjPtr; // Define apointer a undefined object.


extern int  screen_w, screen_h; // Usable size of the screen (to size any windows).

extern int              svargc; // To keep the task arguments number ...
extern char          ** svargv; // ... and the task arguments list.

extern const char    UserEnv[], // Path to DiaViewer setup file(s).
                     UserDir[], // Path directory of the user.
                  DiaEnvFile[]; // DiaViewer Setup main filename.


extern SetUpDynTab   VolToOpen, // Table of volume to open when the application start.
                      WinTable, //  To keep all windows (to create at start time) data.
                     //  The next tables should be replaced by the table WinTable.
                     WinVolume, // Table of assigned Volume for each window.
                     WinSelSli; // Table of values to select previous dispalyed slides.


extern int            slid_dig, // Number of figures for Slide#, 
                      film_dig, // Number of figures for Film#,
                      fold_dig, // Number of figures for Directory/folder,
                      flmn_max, // Maximum value of film number (#),
                      slin_max, // Maximum value of slide number (#),
                      dirslisz, // Max. number of slides/folder in the target volume,
                      dirdirsz; // Max. number of Folder/folder in the target volume.

extern char         * slid_hde, // Slide filename header string,
                    * slid_sep, // Slide filename separator string,
                    * fold_hde, // Folder/Directory filename header string.
                    * fold_tra; // Folder/Directory filename trailer string.

extern char         * tvol_pth, // The Path of target volume.
                    * tvol_nam; // The Name of target volume.


//
// **********************************************************************************
//


//
// class VarString to manage temporary string allocations/free.
//

class VarString {
    
private:

    int       use_, all_, cop_; // Used and allocated size.
    char                * str_; // String pointer.


public:

    VarString( int cp = 0 ) { use_ = all_ = 0; cop_ = cp; str_ = NULL; }
    ~VarString() { if (str_) delete[] str_; }

    char * String() { return str_; }

    char * Size( int );
    void Clear() { if (str_) delete str_; use_ = 0; }

};




//
// **********************************************************************************
//



//
// Class FilePath to manage a dynamic file path.
//

class FilePath {

private:
    char   * path_;
    int    length_,
           allsiz_;
    int   * stack_;
    int   sl_, sp_;
 
public:
    FilePath( int = 0 );
    ~FilePath();

    void         StackSize( int );
    int          Set( const char *, int = 0 );
    int          Rem( int vl = 0 );
    void         Clear();
    const char * Path() { return path_; }
    int          Length() { return length_; }
    
};


extern FilePath     *   CPath; // The CPath common FilePath must be visible out of any class.




//
// **********************************************************************************
//




class ObjTable {
private:
    int           use_, // Used size.
                  all_; // Allocated size.
    ObjPtr      * tab_; // The table of object pointer.

public:
    // Object table creator.
    ObjTable() { use_ = all_ = 0; tab_ = NULL; }
    // Object table destructor.
    ~ObjTable() { if (tab_) delete[] tab_; }

    ObjPtr GetObj( int i ) { return (((i>=0)&&(i<use_))? tab_[i] : NULL); }

    // Routine to get a table access from the specified index.
    ObjPtr * GetObjs( int shf = 0 );

    int    TabUse() { return use_; } // To get the current used table length.

    void           AddObj( ObjPtr ); // Append a new object at the table.

    void        Tamp( int idx = 0 ); // To tamp the object table with some null reference(s).

    void           RemObj( ObjPtr ); // Remove an object from the table.
    void              RemObj( int );

};





//
// **********************************************************************************
//

/*

CLass NSliEntry {
    friend SliEntry;

private:

    SliEntry  * slide_;
    Uint16      flm_nb,
                sli_nb;

public:

    NSliEntry( SliEntry * );

    ~NSliEntry();

    CreateSlide();


};



Class NDirEntry : public DirEntry {
    friend DirEntry;
    
private:

    DirEntry * odir_;
               
public:

    NDirEntry();
    ~NDirEntry();

    CreateDir();


};

*/


//
// Main SetUp definitions
//

extern int  win_dx,     win_dy; // Default Window size.

extern int          SlidePrior, // Flag for scan priority for directories ...
                                // ... (for 0) or for slides if not 0.
                    NativeOFil; // Flag to use native openfile and directory.


//
// Image_Ref definitions
//
extern      int   IIdeOverflow, // Flag for IIde_t overflow.
                  SliCacheSize, // Size to use for Image Cache.
                  IIdeMaxSli,   // Maximum possible # of a slide.
                  IIdeFld[2];   // Sizes of ident fields [0/1] slides/Directories. 

//
// Main Program and GL definitions
//
extern ObjTable   OpenVolTable; // The table of all opened slides volumes.

extern SetUpQueue     VolQueue; // Volume queue (used for user setup.

extern int           PrtFromGL, // Flag to control when we print slide from screen image.
                    SlidePrior, // Flag for scan priority for directories ...
                                // ... (for 0) or for slides if not 0.
                     ScanNoLim, // Scan limited to sub-directories limits (default no).
                    NativeOFil; // Flag to use native openfile and directory.

extern char *      OpenVolPath; // Default Open Volume Path.


//
// Some external function definitions.
//

const char * DLG_DirChooser( const char * msg, const char * fnm ); // From DL module.


#endif

//
// end of <PW-Id>.
//
